#ifndef __SLIDEINSTANCE_H__
#define __SLIDEINSTANCE_H__

#include "slidenode.h"

class CSLIDEWorld;
class CSLIDESurface;
class CSLIDETransform;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEInstance Class
//

class CSLIDEInstanceIter;

class CSLIDEInstance : public CSLIDENode
{
//////////////////////////////////////////////////
// Friends
//
public:
  friend class CSLIDEInstanceIter;

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEInstance();
  virtual ~CSLIDEInstance();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcChildID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual VOID InsertTransform(CSLIDETransform *pCSLIDETransform);
  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEPrimitiveIter *AllocateIter();
  virtual VOID FreeIter(CSLIDEPrimitiveIter *pCSLIDEInstanceIter);

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual CSLIDECamera *FindCamera(CListIterInt *piterIDs, 
				   CListInt *plistCSLIDEInstance);
  virtual CSLIDELight *FindLight(CListIterInt *piterIDs, 
				 CListInt *plistCSLIDEInstance);

  virtual VOID Preprocess();
  virtual VOID PreprocessGraph();

  virtual VOID Update();
  virtual VOID UpdateGraph();
  virtual VOID InsertIntoBound(CBound &bound);

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();
  virtual VOID CalculateBound();

//////////////////////////////////////////////////
// Rendering
//
public:
  virtual VOID Render(const Matrix &mGvp_proj, 
		      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
		      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
		      CBound *pCBoundClip, 
		      CListInt *plistLights,
		      CSLIDESurface *pCSLIDESurface,
		      SLIDEShadingType eShading, 
		      SLIDELODType eLOD);

protected:
  virtual VOID ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface);
  virtual VOID ModifyShadingFlag(SLIDEShadingType &eShading);
  virtual VOID ModifyLODFlag(SLIDELODType &eLOD);
  virtual CBound::BoundIntersectionType CheckBoundReject(const Matrix &m, 
							 CBound *pCBound);
  virtual VOID RenderGeometry(const Matrix &mGvp_proj, 
			      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
			      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
			      CBound *pCBoundClip, 
			      CListInt *plistLights,
			      CSLIDESurface *pCSLIDESurface,
			      SLIDEShadingType eShading, 
			      SLIDELODType eLOD);

public:
  virtual VOID RenderBound(const Matrix &mGvp_proj, const Matrix &mGproj_vrc, 
			   const Matrix &mGvrc_obj);

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  virtual inline VOID SetID(CHAR *pcID);
  virtual inline VOID SetSurfaceID(CHAR *pcSurfaceID);
  virtual inline VOID SetShading(DynInt *pdyniShading);
  virtual inline VOID SetLOD(DynInt *pdyniLOD);

  virtual inline NodeType GetNodeType();
  virtual inline const Matrix &GetTransform();
  virtual inline const Matrix &GetInverse();

  virtual inline BOOL GetTransformDynamic();
  virtual inline VOID SetTransformDynamic(BOOL bTransformDynamic);
  virtual inline BOOL GetBoundEmpty();
  virtual inline VOID SetBoundEmpty(BOOL bBoundEmpty);

  virtual UINT_32 GetTransformCount();

//////////////////////////////////////////////////
// Class Variables
//
protected:

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcChildID;
  CSLIDENode *m_pCSLIDENode;
  CHAR *m_pcSurfaceID;
  CSLIDESurface *m_pCSLIDESurface;
  DynInt m_dyniShading;
  DynInt m_dyniLOD;
  
  CListInt m_listCSLIDETransform;
  Matrix m_mTransform;
  Matrix m_mInverse;

  BOOL m_bTransformDynamic;
  BOOL m_bBoundEmpty;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDEInstance::SetID(CHAR *pcID)
{
  m_pcID = pcID;
}

inline VOID CSLIDEInstance::SetSurfaceID(CHAR *pcSurfaceID)
{
  m_pcSurfaceID = pcSurfaceID;
}

inline VOID CSLIDEInstance::SetShading(DynInt *pdyniShading)
{
  m_dyniShading.Set(pdyniShading);
}

inline VOID CSLIDEInstance::SetLOD(DynInt *pdyniLOD)
{
  m_dyniLOD.Set(pdyniLOD);
}

inline CSLIDENode::NodeType CSLIDEInstance::GetNodeType()
{
  return ntInstance;
}

inline BOOL CSLIDEInstance::GetTransformDynamic()
{
  return m_bTransformDynamic;
}

inline const Matrix &CSLIDEInstance::GetTransform()
{
  return m_mTransform;
}

inline const Matrix &CSLIDEInstance::GetInverse()
{
  return m_mInverse;
}

inline VOID CSLIDEInstance::SetTransformDynamic(BOOL bTransformDynamic)
{
  m_bTransformDynamic = bTransformDynamic;
}

inline BOOL CSLIDEInstance::GetBoundEmpty()
{
  return m_bBoundEmpty;
}

inline VOID CSLIDEInstance::SetBoundEmpty(BOOL bBoundEmpty)
{
  m_bBoundEmpty = bBoundEmpty;
}

//////////////////////////////////////////////////////////////////////
// CSLIDEInstanceIter Class
//

class CSLIDEInstanceIter : public CSLIDEPrimitiveIter
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEInstanceIter();
  virtual ~CSLIDEInstanceIter();
  virtual VOID Init(CSLIDEInstance *pCSLIDEInstance);
  virtual VOID Uninit();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDETransform *PeekFirstCSLIDETransform();
  virtual CSLIDETransform *PeekNextCSLIDETransform();

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CListIterInt m_iter;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//


#endif // __SLIDEINSTANCE_H__
